function P = IOgroup(P,DELTA,C3)
%IOGROUP groups inputs and outputs of P.
% Inputs are:
%   P       -       certain part
%   DELTA   -       uncertain part

%% Define I/O sizes for P
n = length(P.StateName); % size of state vector of P
nw = size(DELTA,1); % size of input unc (w) channels of P
m = length(P.InputName) - nw; % size of input dist (d) channels of P
nv = size(DELTA,2); % size of output unc (v) channels of P
nz = size(C3,1); % size of output est (z) channels of P
p = length(P.OutputName) - nv - nz; % size of output meas (y) channels of P

%% Define state and I/O names for P
% state names of P
nName = cell(1,n);
for k = 1:n
    nName{1,k} = ['x(',num2str(k),')'];
end

% input unc (w) channels of P
nwName = cell(1,nw);
for k = 1:nw
    nwName{1,k} = ['w(',num2str(k),')'];
end

% input dist (d) channels of P
mName = cell(1,m);
for k = 1:m
    mName{1,k} = ['d(',num2str(k),')'];
end

% output unc (v) channels of P
nvName = cell(1,nv);
for k = 1:nv
    nvName{1,k} = ['v(',num2str(k),')'];
end

% output meas (y) channels of P
pName = cell(1,p);
for k = 1:p
    pName{1,k} = ['y(',num2str(k),')'];
end

% output est (z) channels of P
nzName = cell(1,nz);
for k = 1:nz
    nzName{1,k} = ['z(',num2str(k),')'];
end

%% Append output estimation channels to P
% set(P,'c',[P.c;C3],'d',[P.d;zeros(nz,nw+m)]);

%% Assign I/O names and groups
if isa(P,'ss') || isa(P,'tf') || isa(P,'zpk')
    set(P,'InputGroup',struct('Uncertain',1:nw,'Disturbance',nw+1:nw+m),...
        'OutputGroup',struct('Uncertain',1:nv,'Measurement',nv+1:nv+p,...
        'ToBeEstimated',nv+p+1:nv+p+nz),...
        'InputName',[nwName,mName],'OutputName',[nvName,pName,nzName],...
        'StateName',nName);
elseif isa(P,'pss') || isa(P,'plftss') || isa(P,'upss')
    Ptemp = P.Data(:,:,1); % extract first point from grid
    set(Ptemp,'InputGroup',struct('Uncertain',1:nw,'Disturbance',nw+1:nw+m),...
        'OutputGroup',struct('Uncertain',1:nv,'Measurement',nv+1:nv+p,...
        'ToBeEstimated',nv+p+1:nv+p+nz),...
        'InputName',[nwName,mName],'OutputName',[nvName,pName,nzName],...
        'StateName',nName);
    P.Data(:,:,1) = Ptemp; % push back Ptemp to grid
end
end
