%% Robust LPV Estimator Synthesis Using Integral Quadratic Constraints
% Authors: Raghu Venkataraman and Peter Seiler
% Created on Feb 5, 2015.
% Last modified on September 30, 2015.

%% Clean the slate and setup
clear all, close all, clc
warning('off','Control:ltiobject:MultipleChannelMatch');
warning('off','Control:ltiobject:MultipleGroupMatch');

%% Choose IQC mode
IQCMode = 'SingleM'; % Single structured M-matrix is a decision variable.
% IQCMode = 'MultipleLm'; % Multiple lambdas are used to weight the IQCs.
fprintf(['\n - IQC mode is: ',IQCMode,'\n']);

%% Select example mode (choose either 'LTI' or 'LPV')
exampleMode = 'LTI'; % options: 'LTI', 'LPV'
exampleLibrary % <<<<< This file contains plant parameters.
IQCMultDef % <<<<< This file contains basis functions for IQC multipliers.

%% Separate input into certain and uncertain parts
if isa(Punc,'umat') || isa(Punc,'ufrd') || isa(Punc,'uss')...
         || isa(Punc,'pmat')|| isa(Punc,'pss')...
         || isa(Punc,'upmat')|| isa(Punc,'upss')
    [P,DELTA,BLKSTRUCT] = lftdata(Punc); % Punc = Fu(P,DELTA)
end

%% Group inputs and outputs of P
P = IOgroup(P,DELTA,C3);
Punc = lft(DELTA,P); % reform the LFT after naming inputs and outputs

%% Plant extension with PSI filters
[PICT,PSIICT,PSICOLL] = psiFilterICT(P,PSICOLL);

%% Robust synthesis
fprintf('\n ============== ROBUST SYNTHESIS ============== \n');
[F,ubSyn,Mm,lm,Plyap] = robfiltsynEngine(PICT,PSIICT,IQCMode);
fprintf(['\n - [Syn: The L2-L2 norm of lft(DELTA,G) (from disturbances to',...
    '\n\t the estimation error) is %4.2f.\n'],ubSyn);
fprintf('\n - The condition number of Plyap is %e.\n',cond(Plyap));

%% Final analysis
fprintf('\n ============== FINAL ANALYSIS ============== \n');
[GICT,Gnom] = robfiltana(DELTA,P,PICT,F);
[ubAna2,MmAna2,lmAna2,PlyapAna2] = wcgAnalysisEngine(GICT,PSIICT,IQCMode);
fprintf(['\n - [Ana: The L2-L2 norm of lft(DELTA,G) (from disturbances to',...
    '\n\t the estimation error) is %4.2f.\n'],ubAna2);

%% User guide
fprintf('\n ============== USER GUIDE ============== \n');
help userGuide
